// Describes constant data generated by build/embedded_file_gen.py
//
// Note: this functionality could be part of mycpp/Tea

#ifndef CPP_EMBEDDED_FILE_H
#define CPP_EMBEDDED_FILE_H

#include "mycpp/gc_str.h"

struct TextFile {
  const char* rel_path;
  BigStr* contents;  // GC string to avoid copying on return
};

// Linear search; last one has nullptr entries
extern TextFile* gEmbeddedFiles;

#endif  // CPP_EMBEDDED_FILE_H
